/// <reference types="cypress" />
// ***********************************************
// This example commands.ts shows you how to
// create various custom commands and overwrite
// existing commands.
//
// For more comprehensive examples of custom
// commands please read more here:
// https://on.cypress.io/custom-commands
// ***********************************************
//
//
// -- This is a parent command --
// Cypress.Commands.add('login', (email, password) => { ... })
//
//
// -- This is a child command --
// Cypress.Commands.add('drag', { prevSubject: 'element'}, (subject, options) => { ... })
//
//
// -- This is a dual command --
// Cypress.Commands.add('dismiss', { prevSubject: 'optional'}, (subject, options) => { ... })
//
//
// -- This will overwrite an existing command --
// Cypress.Commands.overwrite('visit', (originalFn, url, options) => { ... })
//
// declare global {
//   namespace Cypress {
//     interface Chainable {
//       login(email: string, password: string): Chainable<void>
//       drag(subject: string, options?: Partial<TypeOptions>): Chainable<Element>
//       dismiss(subject: string, options?: Partial<TypeOptions>): Chainable<Element>
//       visit(originalFn: CommandOriginalFn, url: string, options: Partial<VisitOptions>): Chainable<Element>
//     }
//   }
// }


const logout = (): void => {
  cy.log('Logging out by clearing cookies');
  cy.clearAllCookies();
  cy.log('..Waiting a bit after the cookie clear or the logout/next login is flaky');
  cy.wait(3000);
};

Cypress.Commands.add('login', (user?: {username: string; password: string}): void => {
  const _user = user || {
    username: Cypress.env('DEFAULT_USER_NAME'),
    password: Cypress.env('DEFAULT_USER_PASSWORD'),
  };
  cy.log(`Logging in as: username: ${_user.username}, password: ${_user.password}`);
  cy.log('..Logging out (possible) previous user');
  logout();
  cy.visit('/');

  const simpleSamlDomainTop = Cypress.env('SIMPLESAML_DOMAIN').replace(/.*?\./, '');
  const baseUrlDomainTop = new URL(Cypress.config('baseUrl') as string).hostname.replace(/.*?\./, '');
  const baseUrlAndSamlDomainHaveSameTopLevelDomain = simpleSamlDomainTop === baseUrlDomainTop;
  if (baseUrlAndSamlDomainHaveSameTopLevelDomain) {
    cy.get('input#umcLoginUsername').type(_user.username);
    cy.get('input#umcLoginPassword').type(_user.password);
    cy.get('button.umcLoginFormButton').click();
  } else {
    cy.origin(`https://${Cypress.env('SIMPLESAML_DOMAIN')}/`, {args: {__user: _user}}, ({__user}) => {
      cy.get('input#umcLoginUsername').type(__user.username);
      cy.get('input#umcLoginPassword').type(__user.password);
      cy.get('button.umcLoginFormButton').click();
    });
  }
  cy.get('main.listView', {timeout: 10000});
});

Cypress.Commands.add('logout', (): void => {
  logout();
});
